/* jshint node:true */
/* exported DataProvider */
/* global FileManager, DataProviderHelper, ServerPermissionError, ErrorUtility */

// 

var DataProvider = {};

(function () {
    'use strict';

    var __app = (typeof module === 'undefined' ? window : process).__app;

    DataProvider.validateResponse = function validateResponse(xhr) {
        var error = DataProviderHelper.validateRequest(xhr);
        if (!error) {
            var responseText = xhr.responseText;
            if (typeof responseText === 'string') {
                try {
                    var obj = JSON.parse(responseText);
                    if (obj.status === 'error') {
                        if (obj.type === 'permission') {
                            error = new ServerPermissionError(obj.message);
                        }
                    }
                } catch (e) {}
            }
        }
        return error;
    };

    DataProvider.load = function load() {
        return __app.project.data;
    };

    DataProvider.doExport = function doExport(data, callback) {
        var request = {
            'save': {
                'post': {
                    data: JSON.stringify(__app.run(data)),
                    method: data.projectData ? 'save' : 'update'
                },
                'url': __app.backendUrl('export')
            },
            'clear': {
                'post': {},
                'url': __app.backendUrl('clear')
            },
            'errorHandler': DataProvider.validateResponse,
            'encode': true,
            'blob': true
        };
        DataProviderHelper.chunkedRequest(request, callback);
    };

    DataProvider.save = function save(data, callback) {
        DataProvider.doExport(data, callback);
    };

    DataProvider.getMd5Hashes = function getMd5Hashes() {
        return __app.project.hashes;
    };

    DataProvider.getAllCssJsSources = function getAllCssJsSources() {
        return __app.project.cache;
    };

    DataProvider.updatePreviewTheme = function updatePreviewTheme(callback) {
        __app.request('start').done(function updatePreviewThemeSuccess() {
            callback(null);
        }).fail(function updatePreviewThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.makeThemeAsActive = function makeThemeAsActive(callback, theme) {
        theme = theme || '';
        callback(null);
    };

    DataProvider.backToAdmin = function backToAdmin() {
        window.location = __app.config.url.admin;
    };

    DataProvider.getMaxRequestSize = function getMaxRequestSize() {
        return __app.config.env.maxRequestSize;
    };

    DataProvider.getVersion = function getVersion() {
        return '0.0.2';
    };

    DataProvider.reloadThemesInfo = function reloadThemesInfo(callback) {
        __app.request('themes').done(function reloadThemesInfoSuccess(response) {
            __app.config.themes = response.themes;
            callback(null, JSON.stringify({themes: response.themes}));
        }).fail(function reloadThemesInfoRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.getInfo = function getInfo() {
        var originPrefix = typeof module === 'undefined' ?
            window.location.origin + window.location.pathname.replace(/(.+\/).+/, '$1'):
            '';
        var startPage = originPrefix + 'runtime/__preview__/index.html';
        var all = Object.keys(__app.project.model.findAll());
        if (all.length) {
            startPage = originPrefix + __app.project.model.getEntityDataByKey(all[0]).url;
        }
        return {
            cmsName: 'Html',
            cmsVersion: __app.config.version,
            adminPage: __app.config.url.admin,
            startPage: startPage,
            templates: __app.project.model.getUsedTemplates(),
            thumbnails: [{name: 'preview.png', width: 200, height: 200}],
            isThemeActive: __app.project.active,
            themeName: __app.project.name,
            uploadImage: __app.backendUrl('image'),
            uploadTheme: __app.backendUrl('theme'),
            unZip: __app.backendUrl('zipToFso'),
            themeArchiveExt: 'zip',
            themes: __app.config.themes,
            pathToManifest: __app.config.path.manifest || (__app.config.path.editor + '/themler.manifest')
        };
    };

    DataProvider.canRename = function canRename(themeName, callback) {
        __app.request('canRename', {
            themeName: themeName
        }).done(function canRenameSuccess(response) {
            callback(null, response.canRename);
        }).fail(function canRenameRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.rename = function rename(targetName, callback) {
        __app.request('rename', {
            source: __app.project.name,
            target: targetName
        }).done(function renameSuccess() {
            var link = window.location.href.replace(
                new RegExp('themes/' + __app.project.name + '/__editor__'),
                'themes/' + targetName + '/__editor__'
            );
            callback(null, link);
        }).fail(function renameRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.renameTheme = function renameTheme(sourceName, targetName, callback) {
        __app.request('rename', {
            source: sourceName,
            target: targetName
        }).done(function renameThemeSuccess() {
            callback(null);
        }).fail(function renameThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.removeTheme = function removeTheme(sourceName, callback) {
        __app.request('rename', {
            source: sourceName,
            target: ''
        }).done(function removeThemeSuccess() {
            callback(null);
        }).fail(function removeThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.copyTheme = function copyTheme(sourceName, targetName, callback) {
        __app.request('copy', {
            source: sourceName,
            target: targetName
        }).done(function copyThemeSuccess() {
            callback(null);
        }).fail(function copyThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.getTheme = function getTheme(params, callback) {
        var data = {
            sourceName: params.sourceName || '',
            targetName: params.targetName || params.themeName || '',
            editor: !!params.includeEditor
        };

        __app.request('zip', data, 'text').done(function getThemeSuccess(response) {
            callback(null, response);
        }).fail(function getThemeRequestFail(xhr) {
            callback(new Error(xhr.responseText));
        });
    };

    DataProvider.downloadTheme = function downloadTheme(themeName, callback) {
        DataProvider.getTheme({
            sourceName: themeName,
            includeEditor: true
        }, callback);
    };

    DataProvider.setFiles = function setFiles(files, callback) {
        var manifestPath = DataProvider.getInfo().pathToManifest;
        if (files[manifestPath]) {
            var fso = new FileManager();
            fso.write(manifestPath, files[manifestPath]);

            DataProvider.doExport({
                themeFso: fso,
                projectData: __app.project.data
            }, callback);
        } else {
            callback(null);
        }
    };

    DataProvider.zip = function (data, callback) {
        var request = {
            'save': {
                'post': {
                    data: JSON.stringify(data)
                },
                'url': __app.backendUrl('fsoToZip')
            },
            'clear': {
                'post': {},
                'url': __app.backendUrl('clear')
            },
            'errorHandler': DataProvider.validateResponse,
            'encode': true,
            'blob': true
        };
        DataProviderHelper.chunkedRequest(request, callback);
    };

    DataProvider.getPosts = function getPosts(searchObj, callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }
        if (!searchObj || typeof searchObj !== 'object') {
            throw DataProviderHelper.getResultError('Invalid search object');
        }

        var posts = __app.project.model.search(searchObj).map(function (entity) {
            return __app.project.model.getEntityData(entity, ['content']);
        });

        callback(null, {log: null, data: posts});
    };

    DataProvider.getPostChildren = function getPostChildren(id, callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }

        if (!id) {
            throw DataProviderHelper.getResultError('Invalid id');
        }

        callback(null, {log: null, data: __app.project.model.findChildren(id).map(function (entity) {
            return __app.project.model.getEntityData(entity, ['content']);
        })});
    };

    DataProvider.updatePost = function updatePost(postType, post, callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }
        if (!post || typeof post !== 'object' || !post.id) {
            throw DataProviderHelper.getResultError('Invalid post object');
        }

        var updated = __app.project.model.updateEntity(post);

        callback(null, {log: null, data: __app.project.model.getEntityData(updated, ['content'])});
    };

    DataProvider.removePost = function removePost(postType, key, recursive, callback) {
        var list = __app.project.model.removeEntity(key, recursive);
        callback(null, {log: null, data: list});
    };

    DataProvider.copyPost = function copyPost(postType, data, recursive, callback) {
        var list = __app.project.model.cloneEntity(data, recursive);
        callback(null, {log: null, data: list});
    };

    DataProvider.newPost = function newPost(postType, data, callback) {
        var newEntity = __app.project.model.newEntity(postType, data);
        callback(null, {log: null, data: __app.project.model.getEntityData(newEntity, ['content'])});
    };

    if (typeof module !== 'undefined' && module.exports) {
        module.exports = DataProvider;
    }

})();

//